"""
Test the association rule model on a sample set.

@author Norman MacDonald
@date 2010-02-16
"""
import os,sys
from optparse import OptionParser
from pica.Sample import SampleSet, ClassLabelSet
from pica.io.FileIO import FileIO
from pica.AssociationRule import load_rules

	

if __name__ == "__main__":
	parser = OptionParser(version="PICA %prog 1.00")
	parser.add_option("-s","--samples",action="store",dest="input_samples_filename",help="Read samples from FILE",metavar="FILE")
	parser.add_option("-c","--classes",action="store",dest="input_classes_filename",help="Read class labels from FILE",metavar="FILE")
	parser.add_option("-m","--model_filename",action="store",help="use model from FILE for classification",metavar="FILE")
	parser.add_option("-z","--model_accuracy",action="store",help="use model accuracy for classification",metavar="FILE",default="laplace")
	parser.add_option("-t","--targetclass",action="store",dest="target_class",help="Set the target CLASS for testing",metavar="CLASS")
	parser.add_option("-a","--algorithm",action="store",dest="algorithm",help="The algorithm for testing",metavar="ALG", default="cpar.CPARClassifier" )
	parser.add_option("-p","--parameters",action="store",dest="parameters",help="FILE with additional, classifier-specific parameters.",metavar="FILE",default=None)
	(options, args) = parser.parse_args()
	
	
	fileio = FileIO()
	samples = fileio.load_samples(options.input_samples_filename)
	classes = fileio.load_classes(options.input_classes_filename)
	
	rules = load_rules(options.model_filename)
	
	samples.load_class_labels(classes)
	samples.set_current_class(options.target_class)
	samples.hide_nulls(options.target_class)
	modulepath = "pica.classifiers.%s"%(options.algorithm)
	classname = options.algorithm.split(".")[-1]
	ClassifierClass = __import__(modulepath, fromlist=(classname,))
	classifier = ClassifierClass.__dict__[classname](accuracy_measure=options.model_accuracy)
	
	classifier.set_null_flag("NULL")
	classification_results = classifier.test(samples,rules)
	
	print classification_results.print_classification_log()
	print classification_results